@@ -1,8 +1,6 @@
 """Mitype"""
 
 from mitype.app import App
-import mitype.commandline
 
 if __name__ == "__main__":
     OBJ = App()
-    OBJ.start()
