@@ -9,6 +9,7 @@ import mitype.calculations
 import mitype.commandline
 import mitype.keycheck
 
+from datetime import datetime, timedelta 
 
 class App:
 
@@ -18,10 +19,12 @@ class App:
 
         # Start the parser
         self.text = mitype.commandline.main()
-        self.ogtext = self.text
-
         self.tokens = self.text.split()
 
+        # Convert multiple spaces, tabs, newlines to single space
+        self.text = " ".join(self.tokens)
+        self.ogtext = self.text
+
         self.current_word = ""
         self.current_string = ""
 
@@ -35,11 +38,14 @@ class App:
         self.i = 0
         self.mode = 0
 
+        self.win_height = 0
         self.win_width = 0
         self.line_count = 0
 
         self.curr_wpm = 0
 
+        sys.stdout = sys.__stdout__
+
         # Set ESC delay to 0 (default 1 on linux)
         os.environ.setdefault("ESCDELAY", "0")
 
@@ -67,24 +73,24 @@ class App:
 
             # Test mode
             if self.mode == 0:
-                self.TypingMode(win, key)
+                self.typing_mode(win, key)
 
             # Replay mode
             elif self.mode == 1 and mitype.keycheck.is_enter(key):
                 # Start replay if enter key is pressed
-                self.Replay(win)
+                self.replay(win)
 
             # Refresh for changes to show up on window
             win.refresh()
 
-    def TypingMode(self, win, key):
+    def typing_mode(self, win, key):
         # Note start time when first valid key is pressed
         if not self.first_key_pressed and mitype.keycheck.is_valid_initial_key(key):
             self.start_time = time.time()
             self.first_key_pressed = True
 
         if mitype.keycheck.is_resize(key):
-            self.Resize(win)
+            self.resize(win)
 
         if not self.first_key_pressed:
             return
@@ -94,9 +100,21 @@ class App:
         self.key_printer(win, key)
 
     def initialize(self, win):
-        """Initialize curses"""
 
-        curses.initscr()
+        # Find window dimensions
+        self.win_height, self.win_width = self.get_dimensions(win)
+
+        # Adding word wrap to text
+        self.text = self.word_wrap(self.text, self.win_width)
+
+        # Find number of lines required to print text
+        self.line_count = (
+            mitype.calculations.count_lines(self.text, self.win_width) + 2 + 1
+        )
+
+        # If required number of lines are more than the window height, exit
+        if self.line_count > self.win_height:
+            self.size_error()
 
         curses.init_pair(1, curses.COLOR_WHITE, curses.COLOR_GREEN)
         curses.init_pair(2, curses.COLOR_WHITE, curses.COLOR_RED)
@@ -107,17 +125,6 @@ class App:
 
         win.nodelay(True)
 
-        # Find window width
-        self.win_width = self.get_dimensions(win)[1]
-
-        # Adding word wrap to text
-        self.text = self.word_wrap(self.text, self.win_width)
-
-        # Find number of lines required to print text
-        self.line_count = (
-            mitype.calculations.count_lines(self.text, self.win_width) + 2 + 1
-        )
-
         self.setup_print(win)
 
     @staticmethod
@@ -142,7 +149,7 @@ class App:
         """
 
         # Top strip
-        win.addstr(0, int(self.win_width/2)-4, " MITYPE ", curses.color_pair(3))
+        win.addstr(0, int(self.win_width / 2) - 4, " MITYPE ", curses.color_pair(3))
 
         # Print text in BOLD from 3rd line
         win.addstr(2, 0, self.text, curses.A_BOLD)
@@ -164,13 +171,11 @@ class App:
 
         # Handle resizing
         elif mitype.keycheck.is_resize(key):
-            self.Resize(win)
-            pass
-            
+            self.resize(win)
 
         # Check for backspace
         elif mitype.keycheck.is_backspace(key):
-            self.EraseKey()
+            self.erase_key()
 
         # Check for space
         elif key == " ":
@@ -181,27 +186,32 @@ class App:
             self.appendkey(key)
 
         # Update state of window
-        self.UpdateState(win)
+        self.update_state(win)
 
     def keyinput(self, win):
         key = ""
         while key == "":
             try:
-                key = win.getkey()
+                if sys.version_info[0] < 3:
+                    key = win.getkey()
+                else:
+                    key = win.get_wch()
+                    if isinstance(key, int):
+                        if key in (curses.KEY_BACKSPACE, curses.KEY_DC):
+                            return "KEY_BACKSPACE"
+                        if key == curses.KEY_RESIZE:
+                            return "KEY_RESIZE"
             except curses.error:
                 continue
         return key
 
-    def EraseKey(self):
+    def erase_key(self):
         if len(self.current_word) > 0:
-            self.current_word = self.current_word[0: len(
-                self.current_word) - 1]
-            self.current_string = self.current_string[0: len(
-                self.current_string) - 1]
+            self.current_word = self.current_word[0 : len(self.current_word) - 1]
+            self.current_string = self.current_string[0 : len(self.current_string) - 1]
 
     def check_word(self):
-        spc = mitype.calculations.get_spc_count(
-            len(self.current_string), self.text)
+        spc = mitype.calculations.get_spc_count(len(self.current_string), self.text)
         if self.current_word == self.tokens[self.i]:
             self.i += 1
             self.current_word = ""
@@ -214,19 +224,18 @@ class App:
         self.current_word += key
         self.current_string += key
 
-    def UpdateState(self, win):
+    def update_state(self, win):
         win.addstr(self.line_count, 0, " " * self.win_width)
         win.addstr(self.line_count, 0, self.current_word)
 
         win.addstr(2, 0, self.text, curses.A_BOLD)
-        win.addstr(2, 0, self.text[0: len(self.current_string)], curses.A_DIM)
+        win.addstr(2, 0, self.text[0 : len(self.current_string)], curses.A_DIM)
 
-        index = mitype.calculations.change_index(
-            self.current_string, self.text)
+        index = mitype.calculations.change_index(self.current_string, self.text)
         win.addstr(
             2 + index // self.win_width,
             index % self.win_width,
-            self.text[index: len(self.current_string)],
+            self.text[index : len(self.current_string)],
             curses.color_pair(2),
         )
 
@@ -260,53 +269,55 @@ class App:
             self.start_time = 0
         win.refresh()
 
-    def Replay(self, win):
+    def replay(self, win):
 
         win.addstr(self.line_count + 2, 0, " " * self.win_width)
 
         self.setup_print(win)
 
         for j in self.key_strokes:
-
-            time.sleep(j[0])
-
-            self.key_printer(win, j[1])
-
+            timer = time.time()
             key = ""
-
-            try:
-                key = win.getkey()
-            except curses.error:
-                pass
-
-            if mitype.keycheck.is_escape(key):
-                sys.exit(0)
-
+            while time.time() - timer < j[0]:
+                try:
+                    if sys.version_info[0] < 3:
+                        key = win.getkey()
+                    else:
+                        key = win.get_wch()
+                except curses.error:
+                    pass
+                    if mitype.keycheck.is_escape(key):
+                        sys.exit(0)
+       
+            self.key_printer(win, j[1])
             win.refresh()
 
     def word_wrap(self, text, width):
 
-        x = 1
-        while x <= mitype.calculations.count_lines(text, width):
+        for x in range(1, mitype.calculations.count_lines(text, width) + 1):
 
-            if not(x * width >= len(text) or text[x * width - 1] == " "):
+            if not (x * width >= len(text) or text[x * width - 1] == " "):
                 i = x * width - 1
                 while text[i] != " ":
                     i -= 1
-                text = text[:i] + " " * (x * width - i) + text[i + 1:]
+                text = text[:i] + " " * (x * width - i) + text[i + 1 :]
 
-            x += 1
         return text
 
-    def Resize(self, win):
+    def resize(self, win):
         win.clear()
-        self.win_width = self.get_dimensions(win)[1]
+        self.win_height, self.win_width = self.get_dimensions(win)
         self.text = self.word_wrap(self.ogtext, self.win_width)
         self.line_count = (
             mitype.calculations.count_lines(self.text, self.win_width) + 2 + 1
         )
         self.setup_print(win)
-        
-        self.UpdateState(win)
 
-        win.refresh()
\ No newline at end of file
+        self.update_state(win)
+
+        win.refresh()
+
+    def size_error(self):
+        sys.stdout.write("Window too small to print given text")
+        curses.endwin()
+        sys.exit(-1)
